/*
 * HandTracler.h
 *
 *  Created on: Aug 18, 2014
 *      Author: andrey-tsb
 */

#ifndef NUITRACK_HAND_TRACKER_H_
#define NUITRACK_HAND_TRACKER_H_

#include <functional>

#include "nuitrack/modules/Module.h"
#include "nuitrack/types/HandTrackerData.h"

namespace tdv
{
namespace nuitrack
{

/**
 * @brief Represent access to hand tracking.
 * @ingroup HandTracker_group
 */
class HandTracker: public Module
{
public:
    typedef std::shared_ptr<HandTracker> Ptr;

    /*
     *  HandTracker update event
     */
    typedef std::function<void (HandTrackerData::Ptr)> OnUpdate;

    /**
     * @brief Create HandTracker
     *
     * @return new instance of HandTracker
     */
    static HandTracker::Ptr create();

    /**
     * @brief Register OnUpdate callback
     *
     * @param OnUpdate callback
     *
     * @return callback handler
     */
    virtual uint64_t connectOnUpdate(const OnUpdate& callback) = 0;

    /**
     * @brief Unregister OnUpdate callback
     *
     * @param callback handler
     */
    virtual void disconnectOnUpdate(uint64_t handler) = 0;

    /**
     * @brief Get current users hands
     *
     * @return HandTrackerData Ptr
     */
	virtual HandTrackerData::Ptr getData() const = 0;

    /**
     * @brief Get current processing time
     *
     * @return milliseconds
     */
    virtual float getProcessingTime() const = 0;

};

} /* namespace nuitrack */
} /* namespace tdv */

#endif /* NUITRACK_HAND_TRACKER_H_ */
