/*
 * UserDetector.h
 *
 *  Created on: Apr 8, 2014
 *      Author: Alex Desinov
 */

#ifndef NUITRACK_USERTRACKER_H_
#define NUITRACK_USERTRACKER_H_

#include <functional>

#include "nuitrack/modules/Module.h"
#include "nuitrack/types/Export.h"
#include "nuitrack/types/UserFrame.h"

namespace tdv
{
namespace nuitrack
{

/**
 * @brief Represent access user tracking.
 * @ingroup UserTracker_group
 */
class NUITRACK_API UserTracker: public Module
{
public:
	typedef std::shared_ptr<UserTracker> Ptr;

	/**
	 * UserTracker update event.
	 */
	typedef std::function<void (UserFrame::Ptr)> OnUpdate;

	/**
	 * UserTracker new user event.
	 */
	typedef std::function<void (int)> OnNewUser;

	/**
	 * UserTracker lost user event.
	 */
	typedef std::function<void (int)> OnLostUser;

	/**
	 * @brief Create UserTracker.
	 *
	 * @return new instance UserTracker.
	 */
	static UserTracker::Ptr create();

	/**
	 * @brief Register OnUpdate event callback.
	 *
	 * @param callback OnUpdate.
	 *
	 * @return callback handler
	 */
	virtual uint64_t connectOnUpdate(const OnUpdate& callback) = 0;

	/**
	 * @brief Unregister OnUpdate event callback.
	 *
	 * @param callback handler
	 */
	virtual void disconnectOnUpdate(uint64_t handler) = 0;

	/**
	 * @brief Register OnNewUser event callback.
	 *
	 * @param callback OnNewUser
	 *
	 * @return callback handler
	 */
	virtual uint64_t connectOnNewUser(const OnNewUser& callback) = 0;

	/**
	 * @brief Unregister OnNewUser event callback.
	 *
	 * @param callback handler
	 */
	virtual void disconnectOnNewUser(uint64_t handler) = 0;

	/**
	 * Register OnLostUser event callback.
	 *
	 * @param callback OnLostUser
	 *
	 * @return callback handler
	 */
	virtual uint64_t connectOnLostUser(const OnLostUser& callback) = 0;

	/**
	 * @brief Unregister OnLostUser event callback.
	 *
	 * @param callback handler
	 */
	virtual void disconnectOnLostUser(uint64_t handler) = 0;

	/**
	 * @brief Get current UserFrame.
	 *
	 * @return UserFrame Ptr.
	 */
	virtual UserFrame::Ptr getUserFrame() const = 0;

	/**
	 * @brief Get current processing time
	 *
	 * @return milliseconds
	 */
	virtual float getProcessingTime() const = 0;
};

} /* namespace nuitrack */
} /* namespace tdv */

#endif /* NUITRACK_USERTRACKER_H_ */
