/*
 * Error.h
 *
 *  Created on: Jul 22, 2014
 *      Author: Alex Desinov
 */

#ifndef NUITRACK_ERROR_H_
#define NUITRACK_ERROR_H_

#include <stdexcept>

#include "nuitrack/types/Export.h"

namespace tdv
{
namespace nuitrack
{

enum ExceptionType
{
	OK,
	EXCEPTION,
	TERMINATE_EXCEPTION,
	BAD_CONFIG_VALUE_EXCEPTION,
	CONFIG_NOT_FOUND_EXCEPTION,
	MODUDLE_NOT_FOUND_EXCEPTION,
	LICENSE_NOT_ACQUIRED_EXCEPTION,
	MODULE_NOT_INITIALIZED_EXCEPTION,
	MODULE_NOT_STARTED_EXCEPTION
};

class MIDDLEWARE_API Exception : public std::runtime_error
{
public:
	Exception(const std::string& msg = "");

	virtual ExceptionType type() const;
};

class MIDDLEWARE_API TerminateException : public Exception
{
public:
	TerminateException(const std::string& msg = "");

	virtual ExceptionType type() const;
};

class MIDDLEWARE_API BadConfigValueException : public Exception
{
public:
	BadConfigValueException(const std::string& msg = "");

	virtual ExceptionType type() const;
};

class MIDDLEWARE_API ConfigNotFoundException : public TerminateException
{
public:
	ConfigNotFoundException(const std::string& msg = "");

	virtual ExceptionType type() const;
};

class MIDDLEWARE_API ModuleNotFoundException : public TerminateException
{
public:
	ModuleNotFoundException(const std::string& msg = "");

	virtual ExceptionType type() const;
};

class MIDDLEWARE_API LicenseNotAcquiredException : public TerminateException
{
public:
	LicenseNotAcquiredException(const std::string& msg = "");

	virtual ExceptionType type() const;
};

class MIDDLEWARE_API ModuleNotInitializedException : public TerminateException
{
public:
	ModuleNotInitializedException(const std::string& msg = "");

	virtual ExceptionType type() const;
};

class MIDDLEWARE_API ModuleNotStartedException : public TerminateException
{
public:
	ModuleNotStartedException(const std::string& msg = "");

	virtual ExceptionType type() const;
};

}
}

#endif /* NUITRACK_ERROR_H_ */
