/*
 * UserGestures.h
 *
 *  Created on: Jul 29, 2014
 *      Author: Alex Desinov
 */

#ifndef NUITRACK_GESTUREDATA_H_
#define NUITRACK_GESTUREDATA_H_

#include "nuitrack/types/ObjectData.h"
#include "nuitrack/types/Gesture.h"

namespace tdv
{
namespace nuitrack
{

class GestureData: public ObjectData<GestureData>
{
public:
	/**
	 * @brief Get current number of gestures.
	 *
	 * @return Number of gestures.
	 */
	virtual int getNumGestures() const = 0;

	/**
	 * @brief Get current user gestures.
	 *
	 * @return Gestures.
	 */
	virtual const std::vector<Gesture>& getGestures() const = 0;
};

class UserStateData: public ObjectData<UserStateData>
{
public:
	/**
	 * @brief Get current number of gestures.
	 *
	 * @return Number of gestures.
	 */
	virtual int getNumUserStates() const = 0;

	/**
	 * @brief Get user state changes
	 *
	 * @return State changes.
	 */
	virtual const std::vector<UserState>& getUserStates() const = 0;
};

class UserGesturesStateData: public ObjectData<UserGesturesStateData>
{
public:
	/**
	 * @brief Get number of users.
	 *
	 * @return Number of users.
	 */
	virtual int getNumUsers() const = 0;

	/**
	 * @brief Get users and gestures state
	 *
	 * @return User and gesture state for each user.
	 */
	virtual const std::vector<UserGesturesState>& getUserGesturesStates() const = 0;
};

}
}

#endif /* NUITRACK_GESTUREDATA_H_ */
