/*
 * Nuitrack.h
 *
 *  Created on: Apr 4, 2014
 *      Author: Alex Desinov
 */

#ifndef NUITRACK_H_
#define NUITRACK_H_

#include <string>
#include <memory>

#include "nuitrack/types/Export.h"
#include "nuitrack/types/Error.h"

#include "nuitrack/modules/Module.h"
#include "nuitrack/modules/DepthSensor.h"
#include "nuitrack/modules/UserTracker.h"
#include "nuitrack/modules/SkeletonTracker.h"
#include "nuitrack/modules/GestureRecognizer.h"
#include "nuitrack/modules/HandTracker.h"

namespace tdv
{
namespace nuitrack
{

class NUITRACK_API Nuitrack
{
public:
	/**
	 * @brief Initialize Nuitrack.
	 * NOTE: For Android OS config file get from Nuitrack Manager.
	 *
	 * @param config Config file for Nuitrack initialization.
	 */
#if defined ANDROID && !defined ANDROID_NATIVE
	static void init();
#else
	static void init(const std::string& config);
#endif

	/**
	 * @brief Run Nuitrack.
	 * NOTE: Start all Nuitrack modules.
	 */
	static void run();

	/**
	 * @brief Update Nuitrack, call all new events.
	 * NOTE: Non block update Nuitrack, call all new events.
	 */
	static void update();

	/**
	 * @brief Update Nuitrack, call all new events.
	 * NOTE: Non block update Nuitrack, synchronize all events by module.
	 *
	 * @param module Ptr Nuitrack module.
	 */
	static void update(std::shared_ptr<Module> module);

	/**
	 * @brief Wait and update Nuitrack, call all new events.
	 * NOTE: Block update Nuitrack, synchronize all events by module.
	 *
	 * @param module Ptr Nuitrack module.
	 */
	static void waitUpdate(std::shared_ptr<Module> module);

	/**
	 * @brief Release Nuitrack
	 * NOTE: Release and destroy all Nuitrack modules.
	 */
	static void release();

	/**
	 * @brief Configure Nuitrack
	 *
	 * @param key - key in config file
	 * @param value - value by key in config file
	 */
	static void setConfigValue(const std::string& key, const std::string& value);

	/**
	 * @brief Get config value
	 *
	 * @param key - key in config file
	 * @param value - value by key in config file
	 */
	static std::string getConfigValue(const std::string& key);
};

} /* namespace nuitrack */
} /* namespace tdv */

#endif /* NUITRACK_H_ */
