/*
 * Skeleton.h
 *
 *  Created on: Jul 29, 2014
 *      Author: Alex Desinov
 */

#ifndef NUITRACK_SKELETON_H_
#define NUITRACK_SKELETON_H_

#include <vector>

#include "nuitrack/types/Vector3.h"
#include "nuitrack/types/Orientation.h"

namespace tdv
{
namespace nuitrack
{

	/**
	* @brief Joint index meaning(eg. head, hand etc.).
	* @ingroup SkeletonTracker_group
	*/
enum JointType
{
	JOINT_NONE				= 0,
	JOINT_HEAD				= 1,
	JOINT_NECK				= 2,
	JOINT_TORSO				= 3,
	JOINT_WAIST				= 4,

	JOINT_LEFT_COLLAR		= 5,
	JOINT_LEFT_SHOULDER		= 6,
	JOINT_LEFT_ELBOW		= 7,
	JOINT_LEFT_WRIST		= 8,
	JOINT_LEFT_HAND			= 9,
	JOINT_LEFT_FINGERTIP	=10,

	JOINT_RIGHT_COLLAR		=11,
	JOINT_RIGHT_SHOULDER	=12,
	JOINT_RIGHT_ELBOW		=13,
	JOINT_RIGHT_WRIST		=14,
	JOINT_RIGHT_HAND		=15,
	JOINT_RIGHT_FINGERTIP	=16,

	JOINT_LEFT_HIP			=17,
	JOINT_LEFT_KNEE			=18,
	JOINT_LEFT_ANKLE		=19,
	JOINT_LEFT_FOOT			=20,

	JOINT_RIGHT_HIP			=21,
	JOINT_RIGHT_KNEE		=22,
	JOINT_RIGHT_ANKLE		=23,
	JOINT_RIGHT_FOOT		=24
};

/**
* @brief Skeletal joint class.
* @ingroup SkeletonTracker_group
*/
struct Joint
{
	/**
	 * Joint type.
	 */
	JointType type;

	/**
	 * Joint confidence from 0.0 to 1.0. Larger value means more confident joint.
	 */
	float confidence;

	/**
	 * Joint position in real world coordinates.
	 */
	Vector3 real;

	/**
	 * Joint position in normalized projective coordinates
	 * (x, y from 0.0 to 1.0, z is real).
	 */
	Vector3 proj;

	/**
	 * Joint orientation.
	 */
	Orientation orient;
};

/**
* @brief Contents skeleton.
* @ingroup SkeletonTracker_group
*/
struct Skeleton
{
	/**
	* User Id. The same as other(@ref UserTracker, @ref HandTracker, @ref GestureRecognizer) modules uses.
	*/
	int id;

	/**
	* Array of joints. Where each index is ::JointType.
	*/
	std::vector<Joint> joints;
};

}
}

#endif /* NUITRACK_SKELETON_H_ */
